﻿/***********************************************************
 * Purpose:
 *		Intermediate application sample
 *		for use within Application Control Solution
 * 
 * Provided by Arellia Corporation
 ***********************************************************
 * DISCLAIMER:
 * 
 * You may freely use the attached Source Code and Object Code
 * compiled from it, with or without modification, but this 
 * Source Code is provided to you "AS IS" EXCLUSIVE OF ANY 
 * WARRANTY, INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES 
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
 * OR ANY OTHER WARRANTY, WHETHER EXPRESS OR IMPLIED. IN NO
 * EVENT WILL ARELLIA CORPORATION HAVE ANY LIABILITY FOR ANY
 * LOST PROFITS, LOSS OF DATA OR COSTS OF PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES, OR FOR ANY SPECIAL,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, THOSE RESULTING
 * FROM THE USE OF THE SOURCE CODE, OR THE FAILURE OF THE
 * SOURCE CODE TO PERFORM, OR FOR ANY OTHER REASON.
 * 
 * Arellia Corporation reserves the right to include
 * (in portions or entirety) this intermediate application
 * and/or source code in future versions of products.
 *
 **********************************************************/
using System;
using System.Linq;
using System.Security.Principal;
using System.Xml.Linq;
using System.Xml.XPath;
using AeXClient;
using Microsoft.Win32;

namespace Justify
{
	public class ArelliaAgent
	{
		static readonly Guid ApplicationJustificationDataClass = new Guid("EE6EE865-6CDE-439E-B981-F977FB97CA62");

		// private variables used within this class
		XNamespace Ns { get; set; }
		XDocument EventXmlDocument { get; set; }
		IAeXClient ArelliaClient { get; set; }

		// template used to craft the NSE contents
		public string EventTemplate { get; set; }

		// data values used to insert into rows in the dataclass
		public string UserId { get; set; }
		public string UserDomain { get; set; }
		public string Reason { get; set; }
		public string FileHash { get; set; }
		public DateTime EventDate { get; set; }

		// helper combinator
		public string UserName
		{
			get { return UserDomain + "\\" + UserId; }
		}

		/// <summary>
		/// Constructor
		/// </summary>
		public ArelliaAgent()
		{
			// record the date when the application was launched 
			//		(not potentially later, after user has finally given reason)
			EventDate = DateTime.Now;

			var agentType = Type.GetTypeFromProgID("Altiris.AeXNSClient");
			ArelliaClient = Activator.CreateInstance(agentType) as IAeXClient;
			if ( ArelliaClient == null )
				throw new Exception("Error obtaining interface to the base Altiris Agent");

			// get current user identity
			var currentIdentity = WindowsIdentity.GetCurrent();
			if ( currentIdentity == null )
				throw new Exception("Error getting the current user identity.");
			var splits = currentIdentity.Name.Split('\\');
			UserDomain = splits[0];
			UserId = splits[1];
		}

		/// <summary>
		/// Creates an NSE to populate the Evt_Application_Justification event, then sends to the server.
		/// </summary>
		/// <returns></returns>
		public bool SendJustificationEvent()
		{
			// validate supplied values
			ValidateRequiredProperty(EventTemplate, "EventTemplate was not supplied");
			ValidateRequiredProperty(UserId, "UserId was not supplied");
			ValidateRequiredProperty(UserDomain, "UserDomain was not supplied");
			ValidateRequiredProperty(Reason, "Reason was not supplied");

			// compose and send event
			CreateEvent();
			UpdateComputerDetails();
			UpdateUserReferences();
			UpdateFileReferences();
			SendEvent();

			return true;
		}

		/// <summary>
		/// Composes the NSE from the provided template.
		/// </summary>
		private void CreateEvent()
		{
			EventXmlDocument = XDocument.Parse(EventTemplate);
			if ( EventXmlDocument.Root == null )
				throw new Exception("Unable to locate root XML node in supplied template");

			Ns = EventXmlDocument.Root.GetDefaultNamespace();
		}

		/// <summary>
		/// Updates the computer reference and other misc data within the templated NSE
		/// </summary>
		private void UpdateComputerDetails()
		{
			var computerReference = EventXmlDocument.XPathSelectElement("//inventory/resources/resourceItem[@ref=1]");
			if ( computerReference == null )
				throw new Exception("Unable to locate COMPUTER resource reference XML node in supplied template");

			UpdateAttributeValue(computerReference, "guid", ArelliaClient.MachineGuid);

			var row = computerReference.XPathSelectElement("./resourceData/dataClasses/dataClass[@guid='" + ApplicationJustificationDataClass.ToString("B").ToUpper() + "']/row");
			UpdateAttributeValue(row, "c3", EventDate.ToString("s").Replace("T", " "));
			UpdateAttributeValue(row, "c4", EventDate.ToUniversalTime().ToString("s").Replace("T", " "));
			UpdateAttributeValue(row, "c5", Reason);
		}

		/// <summary>
		/// Updates the file references within the templated NSE
		/// </summary>
		private void UpdateFileReferences()
		{
			var fileReference = EventXmlDocument.XPathSelectElement("//inventory/resources/resourceItem[@ref=3]");
			if ( fileReference == null )
				throw new Exception("Unable to locate FILE resource reference XML node in supplied template");

			var fileRtk = fileReference.XPathSelectElement("./resource/resourceTypeKeys/resourceTypeKey");
			if ( fileRtk == null )
				throw new Exception("Unable to locate file reference 'resourceTypeKey' node");

			UpdateAttributeValue(fileRtk, "Hash", FileHash);
		}

		/// <summary>
		/// Updates the user reference details within the templated NSE
		/// </summary>
		private void UpdateUserReferences()
		{
			var userReference = EventXmlDocument.XPathSelectElement("//inventory/resources/resourceItem[@ref=2]");
			if ( userReference == null )
				throw new Exception("Unable to locate USER resource reference XML node in supplied template");

			var userRtk = userReference.XPathSelectElement("./resource/resourceTypeKeys/resourceTypeKey");
			if (userRtk == null )
				throw new Exception("Unable to locate USER 'resourceTypeKey' node");

			var userResItem = userReference.XPathSelectElement("./name");
			if ( userResItem == null )
				userReference.Add(new XElement(Ns + "name", UserId));
			else
				userResItem.Value = UserId;

			UpdateAttributeValue(userRtk, "UserId", UserId);
			UpdateAttributeValue(userRtk, "Domain", UserDomain);

			var userKey = userReference.XPathSelectElement("./resource/resourceKeys/resourceKey[@name='name.domain']");
			if ( userKey == null )
				throw new Exception("Unable to locate USER 'resourceKey' node");

			UpdateAttributeValue(userKey, "value", UserId + "." + UserDomain);
		}

		/// <summary>
		/// Sends the previously composed NSE to the server.
		/// </summary>
		private void SendEvent()
		{
			var eventXml = EventXmlDocument.ToString();
			var priorityPolicy = GetPriorityPolicy();

			var appEvent = new NfySvrClientEvent();
			appEvent.SendEventAsync(ArelliaClient.ServerName, priorityPolicy, eventXml);
		}

		/// <summary>
		/// Reads the primary policy Guid from the registry -- used to handle this NSE once received.
		/// </summary>
		/// <returns></returns>
		private string GetPriorityPolicy()
		{
			using ( var keyAcsServers = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Altiris\Altiris Agent\ApplicationControlAgent\ApplicationControlConfig\Servers") )
			{
				if ( keyAcsServers == null )
					throw new Exception("Unable to access ACS agent server infomation in registry");

				var serverSubKeyNames = keyAcsServers.GetSubKeyNames();
				if ( serverSubKeyNames.Length < 1 )
					throw new Exception("Unable to read ACS agent server keys from registry");

				using ( var primaryServerConfKey = keyAcsServers.OpenSubKey(serverSubKeyNames[0] + @"\Configuration") )
				{
					if ( primaryServerConfKey == null )
						throw new Exception("Unable to read ACS primary server configuration from registry");

					var priorityPolicy = primaryServerConfKey.GetValue("PriorityPolicy") as string;
					if ( string.IsNullOrEmpty(priorityPolicy) )
						throw new Exception("Unable to read ACS primary server priority policy from registry");

					primaryServerConfKey.Close();
					keyAcsServers.Close();

					return priorityPolicy;
				}
			}
		}

		#region Helpers

		protected void ValidateRequiredProperty(string value, string message)
		{
			if ( value == null || value.Trim().Length == 0 )
				throw new Exception(message);
		}

		protected void UpdateAttributeValue(XElement element, string attributeName, string value)
		{
			var userDomainAttribute = element.Attribute(Ns + attributeName);
			if ( userDomainAttribute != null )
				userDomainAttribute.Value = value;
			else
				element.Add(new XAttribute(Ns + attributeName, value));
		}

		#endregion
	}
}
