﻿/***********************************************************
 * Purpose:
 *		Intermediate application sample
 *		for use within Application Control Solution
 * 
 * Provided by Arellia Corporation
 ***********************************************************
 * DISCLAIMER:
 * 
 * You may freely use the attached Source Code and Object Code
 * compiled from it, with or without modification, but this 
 * Source Code is provided to you "AS IS" EXCLUSIVE OF ANY 
 * WARRANTY, INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES 
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
 * OR ANY OTHER WARRANTY, WHETHER EXPRESS OR IMPLIED. IN NO
 * EVENT WILL ARELLIA CORPORATION HAVE ANY LIABILITY FOR ANY
 * LOST PROFITS, LOSS OF DATA OR COSTS OF PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES, OR FOR ANY SPECIAL,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, THOSE RESULTING
 * FROM THE USE OF THE SOURCE CODE, OR THE FAILURE OF THE
 * SOURCE CODE TO PERFORM, OR FOR ANY OTHER REASON.
 * 
 * Arellia Corporation reserves the right to include
 * (in portions or entirety) this intermediate application
 * and/or source code in future versions of products.
 *
 **********************************************************/
using System;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

namespace Justify
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window, INotifyPropertyChanged
	{
		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void NotifyPropertyChanged(string propertyName)
		{
			if ( PropertyChanged != null )
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}

		#endregion

		public ArelliaAgent Agent { get; protected set; }

		private string _userId;
		private string _userDomain;
		private string _processName;
		private int _processId;

		#region Properties

		/// <summary>
		/// Gets or sets the ProcessId.
		/// </summary>
		public int ProcessId
		{
			get { return _processId; }
			protected set
			{
				_processId = value;
				NotifyPropertyChanged("ProcessId");
			}
		}

		/// <summary>
		/// Gets or sets the ProcessName.
		/// </summary>
		public string ProcessName
		{
			get { return _processName; }
			protected set
			{
				_processName = value;
				NotifyPropertyChanged("ProcessName");
			}
		}

		/// <summary>
		/// Gets or sets the UserDomain.
		/// </summary>
		public string UserDomain
		{
			get { return _userDomain; }
			set
			{
				_userDomain = value;
				NotifyPropertyChanged("UserDomain");
			}
		}

		/// <summary>
		/// Gets or sets the UserId.
		/// </summary>
		public string UserId
		{
			get { return _userId; }
			set
			{
				_userId = value;
				NotifyPropertyChanged("UserId");
			}
		}

		#endregion

		/// <summary>
		/// Constructor
		/// </summary>
		public MainWindow()
		{
			Agent = new ArelliaAgent();

			InitializeComponent();
			LayoutRoot.DataContext = this;

			WindowStartupLocation = WindowStartupLocation.CenterScreen;
			Loaded += OnWindowLoaded;
		}

		void OnWindowLoaded(object sender, RoutedEventArgs e)
		{
			var application = (App)Application.Current;
			foreach ( var arg in application.Arguments )
			{
				if ( arg.ToLower().StartsWith("/processid:") || arg.ToLower().StartsWith("-processid:") )
				{
					var splits = arg.Split(':');
					if ( splits.Length > 0 )
					{
						SetProcessId(int.Parse(splits[1]));
						break;
					}
				}
			}

			// ensure we have a process ID
			if ( ProcessId == 0 )
			{
				// no process identifier
				MessageBox.Show("Usage: Justify.exe /ProcessId:N\n\n...where N is the process ID.\n");

				// exit application
				application.IsSuccess = false;
				Close();
				return;
			}

			// set focus to the reason text box
			ReasonText.Focus();
		}

		protected void SetProcessId(int processId)
		{
			if ( processId == 0 )
				throw new Exception("Process ID cannot be zero");

			ProcessId = processId;
			ProcessName = FileUtils.GetNameOfProcess(processId);
		}

		private void OnClickContinue(object sender, RoutedEventArgs e)
		{
			Agent.EventTemplate = XmlResources.EventXmlTemplate;
			Agent.Reason = ReasonText.Text;

			// compute file hash
			var filePath = FileUtils.GetFilePathOfProcess(ProcessId);
			Agent.FileHash = FileUtils.ComputeFileHash(filePath);

			// queue event for delivery by agent
			Agent.SendJustificationEvent();

			// exit application: exit code 0 == continue process
			var application = (App)Application.Current;
			application.IsSuccess = true;
			Close();
		}

		private void OnClickCancel(object sender, RoutedEventArgs e)
		{
			// exit application: discontinue process
			var application = (App)Application.Current;
			application.IsSuccess = false;
			Close();
		}

		private void OnReasonTextChanged(object sender, TextChangedEventArgs e)
		{
			// ensure user has typed in something meaningful (not just whitespace)
			bool isValidReason = ReasonText.Text.Trim().Length > 0;
			ContinueButton.IsEnabled = isValidReason;
		}

	}
}
